@@ -1,46 +1,72 @@
-// Copyright 2021 MaidSafe.net limited.
+// Copyright 2022 MaidSafe.net limited.
 //
 // This SAFE Network Software is licensed to you under The General Public License (GPL), version 3.
 // Unless required by applicable law or agreed to in writing, the SAFE Network Software distributed
 // under the GPL Licence is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
-#![allow(clippy::from_iter_instead_of_collect)]
 
-use serde::{Deserialize, Serialize};
 use std::fmt;
 
+mod amount_secrets;
+mod blst;
 mod builder;
 mod dbc;
 mod dbc_content;
-mod dbc_packet;
-mod dbc_transaction;
 mod error;
+mod genesis;
 mod key_manager;
 mod mint;
+mod owner;
 mod spent_proof;
+mod verification;
+
+#[cfg(feature = "mock")]
+pub mod mock;
+
+// re-export crates used in our public API
+pub use blsttc;
+pub use sn_ringct;
+// note: both sn_ringct::rand and blsttc::rand are
+// exposed in our public API.  Here, by choosing
+// just one, we are making an implicit promise that
+// the two versions will remain compatible, or that
+// our API will reconcile the difference.  We do
+// this knowingly and pledge to uphold that promise.
+pub use sn_ringct::rand;
 
 pub use crate::{
-    builder::{DbcBuilder, Output, ReissueRequestBuilder, TransactionBuilder},
+    amount_secrets::{Amount, AmountSecrets},
+    blst::{BlindingFactor, Commitment, KeyImage},
+    builder::{
+        DbcBuilder, DecoyInput, MlsagMaterial, Output, OutputOwnerMap, RevealedCommitment,
+        RingCtMaterial, RingCtTransaction, TransactionBuilder, TrueInput,
+    },
     dbc::Dbc,
-    dbc_content::{Amount, AmountSecrets, DbcContent},
-    dbc_packet::{DbcPacket, DerivedKeySet},
-    dbc_transaction::DbcTransaction,
+    dbc_content::DbcContent,
     error::{Error, Result},
-    key_manager::{
-        KeyManager, NodeSignature, PublicKey, PublicKeySet, Signature, SimpleKeyManager,
-        SimpleSigner,
-    },
-    mint::{
-        genesis_dbc_input, GenesisDbcShare, MintNode, MintNodeSignatures, ReissueRequest,
-        ReissueShare, ReissueTransaction,
-    },
-    spent_proof::{SpendKey, SpentProof, SpentProofShare},
+    genesis::GenesisMaterial,
+    key_manager::{IndexedSignatureShare, KeyManager, PublicKey, PublicKeySet, Signature},
+    owner::{DerivationIndex, Owner, OwnerOnce},
+    spent_proof::{SpentProof, SpentProofContent, SpentProofShare},
+    verification::TransactionVerifier,
 };
 
-#[derive(Clone, Copy, PartialEq, Eq, Hash, PartialOrd, Ord, Serialize, Deserialize)]
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Serialize};
+
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[derive(Clone, Copy, PartialEq, Eq, Hash, PartialOrd, Ord)]
 pub struct Hash([u8; 32]);
 
+impl Hash {
+    #[allow(clippy::self_named_constructors)]
+    /// sha3 256 hash
+    pub fn hash(input: &[u8]) -> Self {
+        Self::from(sha3_256(input))
+    }
+}
+
 impl From<[u8; 32]> for Hash {
     fn from(val: [u8; 32]) -> Hash {
         Hash(val)
@@ -48,7 +74,6 @@ impl From<[u8; 32]> for Hash {
 }
 
 // Display Hash value as hex in Debug output.  consolidates 36 lines to 3 for pretty output
-// and the hex value is the same as sn_dbc_mint display of DBC IDs.
 impl fmt::Debug for Hash {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_tuple("Hash").field(&hex::encode(self.0)).finish()
@@ -61,59 +86,38 @@ impl AsRef<[u8]> for Hash {
     }
 }
 
-#[cfg(feature = "dkg")]
-use std::convert::TryFrom;
+/// This is a helper module to make it a bit easier
+/// and regular for API callers to instantiate
+/// an Rng when calling sn_dbc methods that require
+/// them.
+pub mod rng {
+    use crate::rand::{
+        rngs::{StdRng, ThreadRng},
+        SeedableRng,
+    };
 
-#[cfg(feature = "dkg")]
-pub fn bls_dkg_id() -> bls_dkg::outcome::Outcome {
-    use std::collections::BTreeSet;
-    use std::iter::FromIterator;
-
-    let owner_name = rand::random();
-    let threshold = 0;
-    let (mut key_gen, proposal) =
-        bls_dkg::KeyGen::initialize(owner_name, threshold, BTreeSet::from_iter([owner_name]))
-            .expect("Failed to init key gen");
-
-    let mut msgs = vec![proposal];
-    while let Some(msg) = msgs.pop() {
-        let response_msgs = key_gen
-            .handle_message(&mut rand::thread_rng(), msg)
-            .expect("Error while generating BLS key");
-
-        msgs.extend(response_msgs);
+    pub fn thread_rng() -> ThreadRng {
+        crate::rand::thread_rng()
     }
 
-    let (_, outcome) = key_gen.generate_keys().unwrap();
-    outcome
-}
-
-#[cfg(feature = "dkg")]
-pub struct DbcHelper {}
-
-#[cfg(feature = "dkg")]
-impl DbcHelper {
-    pub fn decrypt_amount_secrets(
-        owner: &bls_dkg::outcome::Outcome,
-        dbcc: &DbcContent,
-    ) -> Result<AmountSecrets, Error> {
-        let mut shares: std::collections::BTreeMap<usize, bls_dkg::SecretKeyShare> =
-            Default::default();
-        shares.insert(owner.index, owner.secret_key_share.clone());
-
-        AmountSecrets::try_from((&owner.public_key_set, &shares, &dbcc.amount_secrets_cipher))
-    }
-
-    pub fn decrypt_amount(
-        owner: &bls_dkg::outcome::Outcome,
-        dbcc: &DbcContent,
-    ) -> Result<Amount, Error> {
-        Ok(Self::decrypt_amount_secrets(owner, dbcc)?.amount)
+    pub fn from_seed(seed: <StdRng as SeedableRng>::Seed) -> StdRng {
+        StdRng::from_seed(seed)
     }
 }
 
 #[cfg(test)]
-fn sha3_256(input: &[u8]) -> [u8; 32] {
+use {
+    crate::rand::RngCore,
+    blsttc::{SecretKeySet, SecretKeyShare},
+};
+
+#[cfg(test)]
+pub fn bls_dkg_id(rng: &mut impl RngCore) -> (PublicKeySet, SecretKeyShare, usize) {
+    let sks = SecretKeySet::random(0, rng);
+    (sks.public_keys(), sks.secret_key_share(0), 0)
+}
+
+pub(crate) fn sha3_256(input: &[u8]) -> [u8; 32] {
     use tiny_keccak::{Hasher, Sha3};
 
     let mut sha3 = Sha3::v256();
@@ -129,8 +133,11 @@ mod tests {
     use core::num::NonZeroU8;
     use quickcheck::{Arbitrary, Gen};
 
+    pub const STD_DECOYS_PER_INPUT: usize = 3;
+    pub const STD_DECOYS_TO_FETCH: usize = 100;
+
     #[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
-    pub struct TinyInt(u8);
+    pub struct TinyInt(pub u8);
 
     impl TinyInt {
         pub fn coerce<T: From<u8>>(self) -> T {
@@ -155,7 +162,7 @@ mod tests {
     }
 
     #[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
-    pub struct NonZeroTinyInt(NonZeroU8);
+    pub struct NonZeroTinyInt(pub NonZeroU8);
 
     impl NonZeroTinyInt {
         pub fn coerce<T: From<u8>>(self) -> T {
@@ -188,7 +195,7 @@ mod tests {
     }
 
     #[derive(Clone, PartialEq, Eq, PartialOrd, Ord)]
-    pub struct TinyVec<T>(Vec<T>);
+    pub struct TinyVec<T>(pub Vec<T>);
 
     impl<T> TinyVec<T> {
         pub fn into_iter(self) -> impl Iterator<Item = T> {
